package com.example.nto.controller;

import com.example.nto.Utils;
import com.example.nto.entity.Employee;
import com.example.nto.entity.ResponseData;
import com.example.nto.entity.Visits;
import com.example.nto.service.EmployeeService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Optional;

@RestController
@RequestMapping("/api/admin")
public class AdminController {
    @Autowired
    private EmployeeService employeeService;

    private Utils utils = new Utils();

    @GetMapping("/employees")
    public List<Employee> getEmployees() {
        return employeeService.getAllEmployees();
    }

    @GetMapping("/{username}/info")
    public ResponseEntity<?> getEmployeeInfo(@PathVariable("username") String username) {
        Optional<Employee> employee = employeeService.findByLogin(username);
        if (employee.isEmpty()) {
            return utils.NotFound();
        }

        return new ResponseEntity<>(employee.get(), HttpStatus.OK);
    }

    @GetMapping("/{username}/lastVisit")
    public ResponseEntity<?> getEmployeeLastVisit(@PathVariable("username") String username) {
        Optional<Employee> employee = employeeService.findByLogin(username);
        if (employee.isEmpty()) {
            return utils.NotFound();
        }

        return new ResponseEntity<>(employee.get(), HttpStatus.OK);
    }

    @GetMapping("/{username}/visits")
    public ResponseEntity<?> getEmployeeVisits(@PathVariable("username") String username) {
        if (!employeeService.existsByLogin(username)) {
            return utils.NotFound();
        }

        Optional<List<Visits>> visits = Optional.ofNullable(employeeService.getEmployeeVisits(username));

        return new ResponseEntity<>(visits.get(), HttpStatus.OK);
    }
}
